@@ -23,8 +23,8 @@
 #pragma once
 
 #include "defaultdevice.h"
-
-#include <libnova/ln_types.h>
+#include "libastro.h"
+#include "inditimer.h"
 
 #include <string>
 
@@ -116,6 +116,13 @@ class Dome : public DefaultDevice
             SHUTTER_CLOSE /*!< Close Shutter */
         } ShutterOperation;
 
+        /*! Mount Locking Policy */
+        enum MountLockingPolicy
+        {
+            MOUNT_IGNORED,      /*!< Mount is ignored. Dome can park or unpark irrespective of mount parking status */
+            MOUNT_LOCKS,        /*!< Mount Locks. Dome can park if mount is completely parked first. */
+        };
+
         /** \typedef DomeState
                 \brief Dome status
             */
@@ -128,6 +135,8 @@ class Dome : public DefaultDevice
             DOME_UNPARKING, /*!< Dome is unparking */
             DOME_PARKED,    /*!< Dome is parked */
             DOME_UNPARKED,  /*!< Dome is unparked */
+            DOME_UNKNOWN,   /*!< Dome state is known */
+            DOME_ERROR,     /*!< Dome has errors */
         } DomeState;
 
         /** \typedef ShutterStatus
@@ -135,22 +144,23 @@ class Dome : public DefaultDevice
             */
         typedef enum
         {
-            SHUTTER_OPENED, /*!< Shutter is open */
-            SHUTTER_CLOSED, /*!< Shutter is closed */
-            SHUTTER_MOVING, /*!< Shutter is in motion */
-            SHUTTER_UNKNOWN /*!< Shutter status is unknown */
-        } ShutterStatus;
+            SHUTTER_OPENED,     /*!< Shutter is open */
+            SHUTTER_CLOSED,     /*!< Shutter is closed */
+            SHUTTER_MOVING,     /*!< Shutter in motion (opening or closing) */
+            SHUTTER_UNKNOWN,    /*!< Shutter status is unknown */
+            SHUTTER_ERROR       /*!< Shutter status is unknown */
+        } ShutterState;
 
         enum
         {
-            DOME_CAN_ABORT    = 1 << 0, /*!< Can the dome motion be aborted? */
-            DOME_CAN_ABS_MOVE = 1 << 1, /*!< Can the dome move to an absolute azimuth position? */
-            DOME_CAN_REL_MOVE =
-                1
-                << 2, /*!< Can the dome move to a relative position a number of degrees away from current position? Positive degress is Clockwise direction. Negative Degrees is counter clock wise direction */
-            DOME_CAN_PARK    = 1 << 3, /*!< Can the dome park and unpark itself? */
-            DOME_HAS_SHUTTER = 1 << 4, /*!< Does the dome has a shutter than can be opened and closed electronically? */
-            DOME_HAS_VARIABLE_SPEED = 1 << 5 /*!< Can the dome move in different configurable speeds? */
+            DOME_CAN_ABORT          = 1 << 0, /*!< Can the dome motion be aborted? */
+            DOME_CAN_ABS_MOVE       = 1 << 1, /*!< Can the dome move to an absolute azimuth position? */
+            DOME_CAN_REL_MOVE       = 1 << 2, /*!< Can the dome move to a relative position a number of degrees away from current position? Positive degress is Clockwise direction. Negative Degrees is counter clock wise direction */
+            DOME_CAN_PARK           = 1 << 3, /*!< Can the dome park and unpark itself? */
+            DOME_CAN_SYNC           = 1 << 4, /*!< Can the dome sync to arbitrary postion? */
+            DOME_HAS_SHUTTER        = 1 << 5, /*!< Does the dome has a shutter than can be opened and closed electronically? */
+            DOME_HAS_VARIABLE_SPEED = 1 << 6, /*!< Can the dome move in different configurable speeds? */
+            DOME_HAS_BACKLASH       = 1 << 7  /*!< Can the dome compensate for backlash? */
         };
 
         /** \struct DomeConnection
@@ -166,13 +176,13 @@ class Dome : public DefaultDevice
         Dome();
         virtual ~Dome();
 
-        virtual bool initProperties();
-        virtual void ISGetProperties(const char * dev);
-        virtual bool updateProperties();
-        virtual bool ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n);
-        virtual bool ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n);
-        virtual bool ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n);
-        virtual bool ISSnoopDevice(XMLEle * root);
+        virtual bool initProperties() override;
+        virtual void ISGetProperties(const char * dev) override;
+        virtual bool updateProperties() override;
+        virtual bool ISNewNumber(const char * dev, const char * name, double values[], char * names[], int n) override;
+        virtual bool ISNewSwitch(const char * dev, const char * name, ISState * states, char * names[], int n) override;
+        virtual bool ISNewText(const char * dev, const char * name, char * texts[], char * names[], int n) override;
+        virtual bool ISSnoopDevice(XMLEle * root) override;
 
         static void buttonHelper(const char * button_n, ISState state, void * context);
 
@@ -234,6 +244,14 @@ class Dome : public DefaultDevice
             return capability & DOME_CAN_PARK;
         }
 
+        /**
+             * @return True if dome can sync.
+             */
+        bool CanSync()
+        {
+            return capability & DOME_CAN_SYNC;
+        }
+
         /**
              * @return True if dome has controllable shutter door
              */
@@ -250,16 +268,32 @@ class Dome : public DefaultDevice
             return capability & DOME_HAS_VARIABLE_SPEED;
         }
 
+        /**
+         * @return True if the dome supports backlash
+         */
+        bool HasBacklash()
+        {
+            return capability & DOME_HAS_BACKLASH;
+        }
+
         /**
              * @brief isLocked, is the dome currently locked?
              * @return True if lock status equals true, and TelescopeClosedLockTP is Telescope Locks.
              */
         bool isLocked();
 
-        DomeState getDomeState() const;
+        DomeState getDomeState() const
+        {
+            return m_DomeState;
+        }
         void setDomeState(const DomeState &value);
 
-        IPState getWeatherState() const;
+        ShutterState getShutterState() const
+        {
+            return m_ShutterState;
+        }
+        void setShutterState(const ShutterState &value);
+
         IPState getMountState() const;
 
     protected:
@@ -290,6 +324,12 @@ class Dome : public DefaultDevice
             */
         virtual IPState MoveRel(double azDiff);
 
+        /**
+             * \brief Sync sets the dome current azimuth as the supplied azimuth position
+             * \return True if sync is successful, false otherwise.
+             */
+        virtual bool Sync(double az);
+
         /**
              * \brief Abort all dome motion
              * \return True if abort is successful, false otherwise.
@@ -311,6 +351,20 @@ class Dome : public DefaultDevice
              */
         virtual IPState UnPark();
 
+        /**
+         * @brief SetBacklash Set the dome backlash compensation value
+         * @param steps value in absolute steps to compensate
+         * @return True if successful, false otherwise.
+         */
+        virtual bool SetBacklash(int32_t steps);
+
+        /**
+         * @brief SetBacklashEnabled Enables or disables the dome backlash compensation
+         * @param enable flag to enable or disable backlash compensation
+         * @return True if successful, false otherwise.
+         */
+        virtual bool SetBacklashEnabled(bool enabled);
+
         /**
              * \brief Open or Close shutter
              * \param operation Either open or close the shutter.
@@ -324,7 +378,7 @@ class Dome : public DefaultDevice
              * @param status Status of shutter
              * @return Returns string representation of the shutter status
              */
-        const char * GetShutterStatusString(ShutterStatus status);
+        const char * GetShutterStatusString(ShutterState status);
 
         /**
              * \brief setParkDataType Sets the type of parking data stored in the park data file and presented to the user.
@@ -451,7 +505,7 @@ class Dome : public DefaultDevice
              * @param fp pointer to configuration file
              * @return true if successful, false otherwise.
              */
-        virtual bool saveConfigItems(FILE * fp);
+        virtual bool saveConfigItems(FILE * fp) override;
 
         /**
              * @brief updateCoords updates the horizontal coordinates (Az & Alt) of the mount from the snooped RA, DEC and observer's location.
@@ -488,6 +542,9 @@ class Dome : public DefaultDevice
         INumberVectorProperty DomeParamNP;
         INumber DomeParamN[1];
 
+        INumberVectorProperty DomeSyncNP;
+        INumber DomeSyncN[1];
+
         ISwitchVectorProperty DomeShutterSP;
         ISwitch DomeShutterS[2];
 
@@ -500,19 +557,27 @@ class Dome : public DefaultDevice
         ISwitch ParkOptionS[3];
         ISwitchVectorProperty ParkOptionSP;
 
-        ISwitch AutoParkS[2];
-        ISwitchVectorProperty AutoParkSP;
+        //        ISwitch AutoParkS[2];
+        //        ISwitchVectorProperty AutoParkSP;
 
         uint32_t capability;
-        ShutterStatus shutterState;
         DomeParkData parkDataType;
 
         ITextVectorProperty ActiveDeviceTP;
-        IText ActiveDeviceT[2] {};
+        IText ActiveDeviceT[1] {};
 
         // Switch to lock id mount is unparked
-        ISwitchVectorProperty TelescopeClosedLockTP;
-        ISwitch TelescopeClosedLockT[2];
+        ISwitchVectorProperty MountPolicySP;
+        ISwitch MountPolicyS[2];
+
+        // Shutter control on Park/Unpark
+        ISwitchVectorProperty ShutterParkPolicySP;
+        ISwitch ShutterParkPolicyS[2];
+        enum
+        {
+            SHUTTER_CLOSE_ON_PARK,
+            SHUTTER_OPEN_ON_UNPARK,
+        };
 
         INumber PresetN[3];
         INumberVectorProperty PresetNP;
@@ -520,11 +585,32 @@ class Dome : public DefaultDevice
         ISwitchVectorProperty PresetGotoSP;
         INumber DomeMeasurementsN[6];
         INumberVectorProperty DomeMeasurementsNP;
+
         ISwitchVectorProperty OTASideSP;
-        ISwitch OTASideS[2];
+        ISwitch OTASideS[5];
+        // 0 is East, 1 is West, 2 is as reported by mout, 3 as deducted by Hour Angle
+        // 4 ignore pier side and perform as in a fork mount
+        enum
+        {
+            DM_OTA_SIDE_EAST,
+            DM_OTA_SIDE_WEST,
+            DM_OTA_SIDE_MOUNT,
+            DM_OTA_SIDE_HA,
+            DM_OTA_SIDE_IGNORE
+        };
+
+        int mountOTASide = 0; // Side of the telescope with respect of the mount, 1: west, -1: east, 0 not reported
         ISwitchVectorProperty DomeAutoSyncSP;
         ISwitch DomeAutoSyncS[2];
 
+        // Backlash toogle
+        ISwitchVectorProperty DomeBacklashSP;
+        ISwitch DomeBacklashS[2];
+
+        // Backlash steps
+        INumberVectorProperty DomeBacklashNP;
+        INumber DomeBacklashN[1];
+
         double prev_az, prev_alt, prev_ra, prev_dec;
 
         // For Serial and TCP connections
@@ -534,19 +620,18 @@ class Dome : public DefaultDevice
         Connection::TCP * tcpConnection       = nullptr;
 
         // States
-        DomeState domeState;
-        IPState mountState;
-        IPState weatherState;
+        DomeState m_DomeState;
+        ShutterState m_ShutterState;
+        IPState m_MountState;
 
         // Observer geographic coords. Snooped from mount driver.
-        struct ln_lnlat_posn observer;
+        IGeographicCoordinates observer;
         // Do we have valid geographic coords from mount driver?
         bool HaveLatLong = false;
 
-
         // Mount horizontal and equatorial coords. Snoops from mount driver.
-        struct ln_hrz_posn mountHoriztonalCoords;
-        struct ln_equ_posn mountEquatorialCoords;
+        INDI::IHorizontalCoordinates mountHoriztonalCoords;
+        INDI::IEquatorialCoordinates mountEquatorialCoords;
         // Do we have valid coords from mount driver?
         bool HaveRaDec = false;
 
@@ -576,9 +661,12 @@ class Dome : public DefaultDevice
         bool IsParked = false;
         bool IsMountParked = false;
         bool IsLocked = true;
+        bool AutoSyncWarning = false;
 
         const char * ParkDeviceName;
         const std::string ParkDataFileName;
+        INDI::Timer m_MountUpdateTimer;
+        //int m_HorizontalUpdateTimerID { -1 };
         XMLEle * ParkdataXmlRoot, *ParkdeviceXml, *ParkstatusXml, *ParkpositionXml, *ParkpositionAxis1Xml;
 
         double Axis1ParkPosition;
@@ -586,6 +674,9 @@ class Dome : public DefaultDevice
 
         bool callHandshake();
         uint8_t domeConnection = CONNECTION_SERIAL | CONNECTION_TCP;
+
+        // How often we update horizontal coordinates (10 seconds).
+        static constexpr uint32_t HORZ_UPDATE_TIMER { 10000 };
 };
 
 }
